package cz.muni.fi.pa165.config;

import io.swagger.v3.oas.models.OpenAPI;
import io.swagger.v3.oas.models.links.Link;
import io.swagger.v3.oas.models.media.Content;
import io.swagger.v3.oas.models.media.MediaType;
import io.swagger.v3.oas.models.media.Schema;
import io.swagger.v3.oas.models.responses.ApiResponse;
import org.springdoc.core.customizers.OpenApiCustomizer;
import org.springdoc.core.models.GroupedOpenApi;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

@Configuration
public class ApiConfig {

    @SuppressWarnings("Convert2Lambda")
    @Bean
    public OpenApiCustomizer openAPICustomizer() {
        return new OpenApiCustomizer() {
            @Override
            public void customise(OpenAPI openApi) {
                openApi.getComponents().addResponses("SingleMovieResponse",
                        new ApiResponse()
                                .description("response containing a single message")
                                .content(new Content().addMediaType(
                                                "application/json",
                                                new MediaType().schema(new Schema<>().$ref("#/components/schemas/MovieBasicDto"))
                                        )
                                )
                                .addLink("link_to_getMessage",
                                        new Link()
                                                .operationId("getMessage")
                                                .addParameter("id", "$response.body#/id")
                                                .description("""
                                                        The `id` value returned in the response can be used as
                                                        the `id` parameter in `GET /message/{id}`.
                                                        """)
                                )
                );
            }
        };
    }
}
