package cz.muni.fi.pa165.config;

import cz.muni.fi.pa165.data.model.Genre;
import cz.muni.fi.pa165.data.model.Image;
import cz.muni.fi.pa165.data.model.Movie;
import cz.muni.fi.pa165.data.model.MovieStakeHolder;
import cz.muni.fi.pa165.data.repository.GenreRepository;
import cz.muni.fi.pa165.data.repository.ImageRepository;
import cz.muni.fi.pa165.data.repository.MovieRepository;
import cz.muni.fi.pa165.data.repository.StakeHolderRepository;
import jakarta.annotation.PostConstruct;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

@Transactional
@Service
public class InsertInitialDataService {

    private final MovieRepository movieRepository;
    private final StakeHolderRepository stakeHolderRepository;
    private final GenreRepository genreRepository;
    private final ImageRepository imageRepository;

    @Autowired
    public InsertInitialDataService(MovieRepository movieRepository, StakeHolderRepository stakeHolderRepository, GenreRepository categoryRepository, ImageRepository imageRepository) {
        this.movieRepository = movieRepository;
        this.stakeHolderRepository = stakeHolderRepository;
        this.genreRepository = categoryRepository;
        this.imageRepository = imageRepository;
    }

    @PostConstruct
    public void insertInitialData() {
        insertAMovie();
        insertExtraMovieStakeHolders();
        insertExtraImages();
        insertExtraGenres();
    }

    @Transactional(isolation = Isolation.SERIALIZABLE)
    void insertAMovie() {
        MovieStakeHolder director = new MovieStakeHolder();
        director.setName("Francis");
        director.setSurname("Coppola");
        stakeHolderRepository.save(director);

        MovieStakeHolder actor = new MovieStakeHolder();
        actor.setName("Al");
        actor.setSurname("Pacino");
        stakeHolderRepository.flush();

        Genre genre = new Genre();
        genre.setName("Drama");

        Image image1 = new Image();
        image1.setUrl("www.random.com/image1");
        image1.setDescription("Nice title image");
        imageRepository.save(image1);

        Image image2 = new Image();
        image2.setUrl("www.random.com/image2");
        image2.setDescription("Nice image");
        imageRepository.save(image2);

        Set<Genre> categories = new HashSet(Arrays.asList(genre));
        Set<Image> images = new HashSet(Arrays.asList(image1, image2));

        Movie movie = new Movie();
        movie.setTitle("The Godfather");
        movie.setYear(2009);
        movie.setDirector(director);
        movie.addActor(actor);
        movie.setImages(images);
        movie.addGenre(genre);
        movieRepository.save(movie);

    }

    public void insertExtraMovieStakeHolders() {
        MovieStakeHolder actor = new MovieStakeHolder();
        actor.setName("Marlon");
        actor.setSurname("Brando");
        stakeHolderRepository.save(actor);

        MovieStakeHolder actor1 = new MovieStakeHolder();
        actor1.setName("Jason");
        actor1.setSurname("Momoa");
        stakeHolderRepository.save(actor1);

        MovieStakeHolder actor2 = new MovieStakeHolder();
        actor2.setName("Amber");
        actor2.setSurname("Heard");
        stakeHolderRepository.save(actor2);


    }

    public void insertExtraImages() {
        Image image1 = new Image();
        image1.setUrl("www.random.com/image3");
        image1.setDescription("Ugly title image");

        Image image2 = new Image();
        image2.setUrl("www.random.com/image4");
        image2.setDescription("Low res image");

        imageRepository.save(image1);
        imageRepository.save(image2);
    }

    private void insertExtraGenres() {
        Genre genre1 = new Genre();
        genre1.setName("Thriller");
        genreRepository.save(genre1);

        Genre genre2 = new Genre();
        genre2.setName("Comedy");
        genreRepository.save(genre2);

        Genre genre3 = new Genre();
        genre3.setName("Romance");
        genreRepository.save(genre3);
    }

}
