package cz.muni.fi.pa165.service;

import cz.muni.fi.pa165.data.exception.EntityNotFoundException;
import cz.muni.fi.pa165.data.exception.EntityValidationException;
import cz.muni.fi.pa165.data.model.Rating;
import cz.muni.fi.pa165.data.repository.CriterionRepository;
import cz.muni.fi.pa165.data.repository.MovieRepository;
import cz.muni.fi.pa165.data.repository.RatingRepository;
import cz.muni.fi.pa165.data.repository.UserRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

import java.util.Optional;

@Service
public class RatingService {

    private final RatingRepository ratingRepository;
    private final CriterionRepository criterionRepository;
    private final MovieRepository movieRepository;
    private final UserRepository userRepository;

    @Autowired
    public RatingService(RatingRepository ratingRepository, CriterionRepository criterionRepository, MovieRepository movieRepository, UserRepository userRepository) {
        this.ratingRepository = ratingRepository;
        this.criterionRepository = criterionRepository;
        this.movieRepository = movieRepository;
        this.userRepository = userRepository;
    }

    public Page<Rating> getAllByCriteria(Optional<Integer> userId, Optional<Integer> movieId, Optional<Integer> criterionId, Pageable pageable) {
        return ratingRepository.findAllByUserIdAndMovieIdAndCriterionId(userId, movieId, criterionId, pageable);
    }

    public Rating upsert(Rating rating) throws EntityNotFoundException, EntityValidationException {
        if (rating.getValue() < 0 || rating.getValue() > 10) {
            throw new EntityValidationException("Rating must be between 0 and 10.");
        }

        if (userRepository.findById(rating.getUser().getId()).isEmpty()) {
            throw new EntityNotFoundException("User", rating.getUser().getId());
        }

        if (movieRepository.findById(rating.getMovie().getId()).isEmpty()) {
            throw new EntityNotFoundException("Movie", rating.getMovie().getId());
        }

        if (criterionRepository.findById(rating.getCriterion().getId()).isEmpty()) {
            throw new EntityNotFoundException("Criterion", rating.getCriterion().getId());
        }

        Optional<Rating> exists = ratingRepository.findByUserIdAndMovieIdAndCriterionId(rating.getUser().getId(), rating.getMovie().getId(), rating.getCriterion().getId());
        exists.ifPresent(value -> rating.setId(value.getId()));

        return ratingRepository.save(rating);
    }

    public Rating delete(int userId, int movieId, int criterionId) throws EntityNotFoundException {
        Rating entity = ratingRepository.findByUserIdAndMovieIdAndCriterionId(userId, movieId, criterionId)
                .orElseThrow(() -> new EntityNotFoundException("Matching rating was not found."));

        ratingRepository.delete(entity);

        return entity;
    }
}
