package cz.muni.fi.pa165.rest;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import org.junit.jupiter.api.Test;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.web.servlet.MockMvc;

@SpringBootTest
@AutoConfigureMockMvc
@DirtiesContext(classMode = DirtiesContext.ClassMode.BEFORE_EACH_TEST_METHOD)
class RatingControllerTest {

    @Autowired
    private MockMvc mockMvc;

    @Test
    void getAllTest() throws Exception {
        this.mockMvc.perform(get("/api/rating"))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().json("""
                        [
                          	{
                          		"value": 1,
                          		"criterionId": 1,
                          		"movieId": 1,
                          		"userId": 1
                          	},
                          	{
                          		"value": 5,
                          		"criterionId": 2,
                          		"movieId": 2,
                          		"userId": 2
                          	},
                          	{
                          		"value": 10,
                          		"criterionId": 3,
                          		"movieId": 3,
                          		"userId": 3
                          	},
                          	{
                          		"value": 5,
                          		"criterionId": 2,
                          		"movieId": 1,
                          		"userId": 1
                          	},
                          	{
                          		"value": 6,
                          		"criterionId": 1,
                          		"movieId": 2,
                          		"userId": 2
                          	},
                          	{
                          		"value": 1,
                          		"criterionId": 3,
                          		"movieId": 3,
                          		"userId": 3
                          	},
                          	{
                          		"value": 6,
                          		"criterionId": 3,
                          		"movieId": 1,
                          		"userId": 1
                          	},
                          	{
                          		"value": 3,
                          		"criterionId": 2,
                          		"movieId": 2,
                          		"userId": 2
                          	},
                          	{
                          		"value": 0,
                          		"criterionId": 1,
                          		"movieId": 3,
                          		"userId": 3
                          	}
                          ]
                        """));
    }

    @Test
    void getAllByUserTest() throws Exception {
        this.mockMvc.perform(get("/api/rating?userId=1"))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().json("""
                        [
                         	{
                         		"value": 1,
                         		"criterionId": 1,
                         		"movieId": 1,
                         		"userId": 1
                         	},
                         	{
                         		"value": 5,
                         		"criterionId": 2,
                         		"movieId": 1,
                         		"userId": 1
                         	},
                         	{
                         		"value": 6,
                         		"criterionId": 3,
                         		"movieId": 1,
                         		"userId": 1
                         	}
                         ]
                        """));
    }

    @Test
    void getAllByMovieTest() throws Exception {
        this.mockMvc.perform(get("/api/rating?movieId=1"))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().json("""
                        [
                          	{
                          		"value": 1,
                          		"criterionId": 1,
                          		"movieId": 1,
                          		"userId": 1
                          	},
                          	{
                          		"value": 5,
                          		"criterionId": 2,
                          		"movieId": 1,
                          		"userId": 1
                          	},
                          	{
                          		"value": 6,
                          		"criterionId": 3,
                          		"movieId": 1,
                          		"userId": 1
                          	}
                          ]
                        """));
    }

    @Test
    void getAllByCriterionTest() throws Exception {
        this.mockMvc.perform(get("/api/rating?criterionId=1"))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().json("""
                        [
                           	{
                           		"value": 1,
                           		"criterionId": 1,
                           		"movieId": 1,
                           		"userId": 1
                           	},
                           	{
                           		"value": 6,
                           		"criterionId": 1,
                           		"movieId": 2,
                           		"userId": 2
                           	},
                           	{
                           		"value": 0,
                           		"criterionId": 1,
                           		"movieId": 3,
                           		"userId": 3
                           	}
                           ]
                        """));
    }

    @Test
    void getAllByMovieCriterionUserTest() throws Exception {
        this.mockMvc.perform(get("/api/rating?userId=1&movieId=1&criterionId=1"))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().json("""
                        [
                         	{
                         		"value": 1,
                         		"criterionId": 1,
                         		"movieId": 1,
                         		"userId": 1
                         	}
                         ]
                        """));
    }

    @Test
    void deleteTest() throws Exception {
        this.mockMvc.perform(delete("/api/rating?userId=1&movieId=1&criterionId=1"))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().json("""
                        {
                           "value": 1,
                           "criterionId": 1,
                           "movieId": 1,
                           "userId": 1
                        }
                        """));
    }

    @Test
    void addTest() throws Exception {
        this.mockMvc.perform(put("/api/rating").content("""
                        {
                           "value": 5,
                           "criterionId": 1,
                           "movieId": 1,
                           "userId": 1
                        }
                        """)
                        .contentType(MediaType.APPLICATION_JSON_VALUE))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().json("""
                        {
                           "value": 5,
                           "criterionId": 1,
                           "movieId": 1,
                           "userId": 1
                         }
                        """));
    }

    @Test
    void editTest() throws Exception {
        this.mockMvc.perform(put("/api/rating").content("""
                        {
                           "value": 5,
                           "criterionId": 1,
                           "movieId": 1,
                           "userId": 1
                        }
                        """)
                        .contentType(MediaType.APPLICATION_JSON_VALUE))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().json("""
                        {
                           "value": 5,
                           "criterionId": 1,
                           "movieId": 1,
                           "userId": 1
                         }
                        """))
                .andDo(result -> this.mockMvc.perform(
                        put("/api/rating")
                                .content("""
                        {
                           "value": 2,
                           "criterionId": 1,
                           "movieId": 1,
                           "userId": 1
                        }
                        """)
                                .contentType(MediaType.APPLICATION_JSON_VALUE))
                        .andDo(print())
                        .andExpect(status().isOk())
                        .andExpect(content().json("""
                        {
                           "value": 2,
                           "criterionId": 1,
                           "movieId": 1,
                           "userId": 1
                         }
                        """)));
    }
}
