package cz.muni.fi.pa165.rest;

import cz.muni.fi.pa165.api.auth.login.LoginRequestDto;
import cz.muni.fi.pa165.api.auth.login.LoginResponseDto;
import cz.muni.fi.pa165.api.auth.registration.RegisterRequestDto;
import cz.muni.fi.pa165.api.auth.registration.RegisterResponseDto;
import cz.muni.fi.pa165.facade.AuthFacade;
import io.swagger.v3.oas.annotations.OpenAPIDefinition;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.info.Contact;
import io.swagger.v3.oas.annotations.info.Info;
import io.swagger.v3.oas.annotations.info.License;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import lombok.RequiredArgsConstructor;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@RestController
@OpenAPIDefinition(
        info = @Info(
                title = "Auth management",
                version = "1.0",
                description = "Manage user authorization and authentication",
                contact = @Contact(name = "Petr Wehrenberg", email = "485059@mail.muni.cz"),
                license = @License(name = "Apache 2.0", url = "https://www.apache.org/licenses/LICENSE-2.0.html")
        )
)
@RequestMapping("/api/auth")
@RequiredArgsConstructor
public class AuthRestController {

    private final AuthFacade authFacade;

    @Operation(
            summary = "User registration",
            description = """
                        Register new user. User is authenticated after registration.
                        Returns JWT token for further authentication.
                    """,
            responses = {
                    @ApiResponse(responseCode = "200", content = @Content(
                            schema = @Schema(implementation = RegisterResponseDto.class))
                    )
            }
    )
    @PostMapping("register")
    public ResponseEntity<RegisterResponseDto> register(
            @RequestBody RegisterRequestDto request
    ) {
        var registerRes = authFacade.register(request);
        return ResponseEntity.ok(registerRes);
    }

    @Operation(
            summary = "User login",
            description = """
                        Login user. Returns JWT token for further authentication.
                    """,
            responses = {
                    @ApiResponse(responseCode = "200", content = @Content(
                            schema = @Schema(implementation = LoginResponseDto.class))
                    )
            }
    )
    @PostMapping("login")
    public ResponseEntity<LoginResponseDto> register(
            @RequestBody LoginRequestDto request
    ) {
        var loginRes = authFacade.login(request);
        return ResponseEntity.ok(loginRes);
    }
}
