package cz.muni.fi.pa165.rest;

import cz.muni.fi.pa165.api.user.UserAuthResponseDto;
import cz.muni.fi.pa165.api.user.UserListResponseDto;
import cz.muni.fi.pa165.facade.UserFacade;
import io.swagger.v3.oas.annotations.OpenAPIDefinition;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.info.*;

import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import lombok.RequiredArgsConstructor;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;

@OpenAPIDefinition(
        info = @Info(
                title = "User repository",
                version = "1.0",
                description = "Managing users, authentication and authorization",
                contact = @Contact(name = "Petr Wehrenberg", email = "485059@mail.muni.cz"),
                license = @License(name = "Apache 2.0", url = "https://www.apache.org/licenses/LICENSE-2.0.html")
        )
)
@RestController
@RequiredArgsConstructor
@RequestMapping(path="/api/users")
public class UserRestController {

    private final UserFacade userFacade;

    @Operation(
            summary = "User list",
            description = "Return list of all users.",
            responses = {
                    @ApiResponse(responseCode = "200", content = @Content(
                            schema = @Schema(implementation = UserListResponseDto.class))
                    )
            }
    )
    @GetMapping(path = "")
    public ResponseEntity<UserListResponseDto> getAllUsers() {
        var responseDto = userFacade.getAllUsers();
        return ResponseEntity.ok(responseDto);
    }

    @Operation(
            summary = "Authenticate user",
            description = "Return information about authenticated user",
            responses = {
                    @ApiResponse(responseCode = "200", content = @Content(
                            schema = @Schema(implementation = UserAuthResponseDto.class))
                    )
            }
    )
    @GetMapping(path = "/authenticate")
    public ResponseEntity<UserAuthResponseDto> authenticate() {
        var auth = SecurityContextHolder.getContext().getAuthentication();
        var responseDto = userFacade.getUserByEmail(auth.getName());
        return ResponseEntity.ok(responseDto);
    }
}